package Shape;

import javafx.geometry.Bounds;
import javafx.geometry.Point3D;
import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.paint.Color;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.shape.*;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Random;
import java.util.Set;

public class Obstacle {

    public Obstacle(int WindowSize, double xTranslation, double yTranslation,
                    double zTranslation, int obSize, String shape, Color color
            , boolean immovable){
        this.obSize = obSize/2;
        this.WindowSize = WindowSize;
        this.xTranslation = xTranslation + (double) obSize/2;
        this.yTranslation = yTranslation + (double) obSize/2;
        this.zTranslation = zTranslation + (double) obSize/2;
        this.shape = makeNode(shape, color);
        this.shape.setTranslateX(xTranslation);
        this.shape.setTranslateY(yTranslation);
        this.shape.setTranslateZ(zTranslation);
        this.immovable = immovable;
        shapeBound = this.shape.getBoundsInParent();
    }

    public Node makeNode(String shape, Color color){
        if(shape.equals("Box")){
            Box b = new Box(obSize,obSize,obSize);
            b.setMaterial(new PhongMaterial(color));
            return b;
        } else if (shape.equals("Sphere")){
            Sphere s = new Sphere(obSize);
            s.setMaterial(new PhongMaterial(color));
            return s;
        } else {
            Cylinder c = new Cylinder(obSize,obSize);
            c.setMaterial(new PhongMaterial(color));
            return c;
        }
    }

    public Node getShape(){
        return shape;
    }
    public void addToGroup(Group group){
        group.getChildren().add(shape);
    }
    public String checkCollision(Bounds otherBounds) {
        if (shapeBound.intersects(otherBounds)) {
            if(immovable){
                return "IMMOVABLE";
            }
            double deltaX = Math.min(shapeBound.getMaxX() - otherBounds.getMinX(), otherBounds.getMaxX() - shapeBound.getMinX());
            double deltaY = Math.min(shapeBound.getMaxY() - otherBounds.getMinY(), otherBounds.getMaxY() - shapeBound.getMinY());
            double deltaZ = Math.min(shapeBound.getMaxZ() - otherBounds.getMinZ(), otherBounds.getMaxZ() - shapeBound.getMinZ());

            // Determine minimum overlap
            if (deltaX < deltaY && deltaX < deltaZ) {
                if (shapeBound.getMaxX() > otherBounds.getMaxX()) {
                    return "RIGHT"; //RIGHT HIT
                } else {
                    return "LEFT";
                }
            } else if (deltaY < deltaZ) {
                if (shapeBound.getMaxY() > otherBounds.getMaxY()) {
                    return "TOP";
                } else {
                    return "BOTTOM";
                }
            } else {
                if (shapeBound.getMaxZ() > otherBounds.getMaxZ()) {
                    return "FRONT";
                } else {
                    return "BACK";
                }
            }
        }
        return "NONE";
    }
    boolean immovable;
    int WindowSize = 0;
    double xTranslation, yTranslation, zTranslation;
    Node shape;
    Bounds shapeBound;

    int obSize = 10;
}
