package Shape;

import Shape.Obstacle;
import javafx.geometry.Bounds;
import javafx.geometry.Point3D;
import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.paint.Color;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.shape.Box;
import javafx.scene.shape.MeshView;

import java.util.*;

public class ObstacleField {

    public ObstacleField(int windowSize, int xTranslation, int yTranslation,
                         int range, int min, int obSize, int objNum,
                         String shape, Color color, boolean immovable){
        this.obSize = obSize/2;
        this.windowSize = windowSize;
        this.xTranslation = xTranslation + obSize/2;
        this.yTranslation = yTranslation + obSize/2;
        zTranslation = obSize/2;

        double x, y, z;
        String posKey;
        for(int i = 0; i<objNum; i++){
            Random rand = new Random();
            do {
                x = rand.nextDouble() * range + min; // Random position for x
                y = rand.nextDouble() * range + min; // Random position for y
                z = rand.nextDouble() * range + min; // Random position for z
                posKey = x + "," + y + "," + z;   // Create a key for the set
            } while (occupiedPositions.contains(posKey)); // Ensure uniqueness

            occupiedPositions.add(posKey); // Add the new position to the set

            Obstacle ob = new Obstacle(this.windowSize, this.xTranslation + x, this.yTranslation +y,
                    this.zTranslation + z, this.obSize, shape, color, immovable);
            obstacles.add(ob);
            obstacleBounds.add(ob.getShape().getBoundsInParent());
        }
    }


    public void addToGroup(Group group){
        for(Obstacle o: obstacles){
            group.getChildren().add(o.getShape());
        }
    }
    public ArrayList<Obstacle> getObstacles() {
        return obstacles;
    }

    public ArrayList<Bounds> getObstacleBounds() {
        return obstacleBounds;
    }

    public List<Object> checkCollision(Bounds otherBounds) {
        for (Obstacle o : obstacles) {
            if(o.shapeBound.intersects(otherBounds)){
                return Arrays.asList(o.checkCollision(otherBounds), o);
            }
        }
        return Arrays.asList("NONE", null); // No collision detected
    }

    public void addToField(Obstacle o){
        obstacles.add(o);
        obstacleBounds.add(o.getShape().getBoundsInParent());
    }

    private int windowSize, obSize;
    private int xTranslation,yTranslation,zTranslation;
    Set<String> occupiedPositions = new HashSet<>();
    ArrayList<Obstacle> obstacles = new ArrayList<Obstacle>();
    ArrayList<Bounds> obstacleBounds = new ArrayList<Bounds>();
}
